import java.awt.*;
import java.util.*;
import java.net.*;

public class ChessCanvas extends java.applet.Applet
{
  // 1000  = not be used or passed
  // 0     = empty square  
  // 1,2   = white & black king
  // 3,4   = white & black queen
  // 5,6   = white & black rook
  // 7,8   = white & black knight
  // 9,10  = white & black bishop
  // 11,12 = white & black pawn
  // 13,14 = white & black chancellor
  // 15,16 = white & black marshall
  //   Bishops conversion rule(must take 1-step orthogonal move at 1st move?):
  // 31 = unconverted bishop. Can move as bishop or wazir. L in setup
  // 32 = - " - Black                                      l in setup
  // 33 = to be converted bishop. Must make a wazir move, then
  //      moves as bishop. White
  // 34 = - " - Black
  int board [][]; // position (i,j)  (a1 = 0,0; b3= 2,3; h8 = 7,7, etc.)
  char sqcolor [][]; // Color of (i,j) positions :
    // W = white, b = yellow, B = gray(dark) R = red, X = black
    // Images for loading
  MediaTracker trk;
  Image WhitePawn, BlackPawn,       WhiteKing, BlackKing;
  Image WhiteQueen, BlackQueen,     WhiteRook, BlackRook;
  Image WhiteBishop, BlackBishop,   WhiteKnight, BlackKnight;
  Image WhiteMarshal, BlackMarshal, WhiteChancellor, BlackChancellor;
  ChessMoveChecker control;
  int sqsize;                    // square size in pixels
  int rows, columns ;            // Number of rows and columns on the board 
  int rowClicked, columnClicked; // Clicked row & column , -1 if not clicked
  int computercolor;             // What color is played by the computer?
  int moveNumber = 0;
  String thesetup;               // Initial board setup
  String thecolors;              // Initial Colors of the board
  String version = "PIXELPUSHER V0.4 CV";
  TextField ta = new TextField(50);  // Area for program messages
  Panel links; 
  Stack stack  = new Stack();    // stack of back move board states


  // ----------------------------------------------------------------------

// Extensions can load options.
public void loadoptions()
{
}

  public void init()
  {
    rows      = Integer.parseInt(getParameter("rows")); 
    columns   = Integer.parseInt(getParameter("columns")); 
    thesetup  = getParameter("setup");
    thecolors = getParameter("colors");
    sqsize    = 32;
    computercolor = 1;           // the computer always plays black
    reset();            // Reset (used for restart too
    LoadImages();
    sqcolor = new char[columns][rows];
    ColorBoard(thecolors);
    resize (sqsize*columns, sqsize*rows+40); // + TextArea
  
    links = new Panel();
    links.setLayout( new BorderLayout ());
    setLayout( new BorderLayout ());
    add ("West", links);

    ta.setFont( new Font ("Helvetica", Font.BOLD, 12)); 
    ta.setEditable(false);  
    ta.setBackground( Color.white);  
    add("South", ta);
    loadoptions();
  }
public void reset ()
  {     if (moveNumber >0) moveNumber--;
        rowClicked = -1;
        board   = new int [columns][rows];         // default is 0 - empty
        ForsytheToBoard();
        control =  new ChessMoveChecker(this);
        setText( (control.lastplayermoved != 0) ? "White move":"Black move");
  }
 public void setText (String text) 
 {  ta.setText  ( (moveNumber > 0 ) ? moveNumber + ". "+text : version );}
 public void paint (Graphics g)
 {
    for (int i = 0; i<columns ; i++)
     for (int j = 0; j<rows ; j++)
	    drawsquare (g, i, j);  // color squares
    for (int i = 0; i<columns ; i++)
     for (int j = 0; j<rows ; j++)
            drawpiece (g,i,j);     // draw pieces
 }
 public void fillSquare (Color c, Graphics g, int i, int j)
  {  g.setColor(c); 
     g.fillRect(i*sqsize, (rows-j-1)* sqsize, sqsize, sqsize);
  }
 public void drawsquare( Graphics g, int i, int j)
 {
  if (i == columnClicked &&
      j == rowClicked)           fillSquare(Color.red, g, i, j);
  else if ((sqcolor[i][j]=='W') 
        || board[i][j] == 1000)  fillSquare(Color.white, g, i, j);
  else if (sqcolor[i][j]=='w')   fillSquare(Color.white, g, i, j);
  else if (sqcolor[i][j]=='b')   fillSquare(Color.yellow,g, i, j);
  else if (sqcolor[i][j]=='B')   fillSquare(Color.gray , g, i, j);
  else if (sqcolor[i][j]=='R')   fillSquare(Color.red ,  g, i, j);
  else if (sqcolor[i][j]=='X')   fillSquare(Color.black, g, i, j);
 }
 public boolean mouseDown (Event e, int x, int y) 
 {     
        if (e.modifiers != 0)      // right mouse button - reset or back move
	  { if (e.clickCount > 1)   // righ mouse button double click - reset
	    {   moveNumber = 0;
   	         reset();
            } else                  // back move 
            {   reset();
  	        if ( !stack.empty() )  // pop previous move state 
                   control.SetBoard(  (BoardState) stack.pop() );
            }
	  }
         else 
              clickProcessing ( x/sqsize, rows - y/sqsize - 1);
        repaint();
        return true;
 }
 void clickProcessing (int i, int j)    // i - column, j - row
 {
   if (board[i][j] == 1000) // non-existing (white) square ...
        return; 
   if (rowClicked == -1) // First click : from where the player should move ?
   {
     if (board[i][j] > 0)  // not empty
       if (control.RightPlayer(i,j))
       { columnClicked = i;
         rowClicked = j;
       }
   }
   else // Second click : to which squre player should move
   {
     // if clicked on same square, clicking undone
     if (i == columnClicked && j == rowClicked)
        rowClicked = -1;
     else if ( control.LegalMove(columnClicked,rowClicked,i,j) &&
               control.RightPlayer(columnClicked,rowClicked) )
        {      // Back move preparation - push current state to stack
          stack.push(new BoardState (control)); 

          control.DoMove (columnClicked,rowClicked,i,j);
          rowClicked = -1;
	  MakeComputerMove();
        }
   }
 }
 // drawpiece on position (i,j)
 public void drawpiece (Graphics g, int i, int j)
  {
     Image image = null;   // image on position i,j
  
     if      ( board [i][j] == 1 )  image = WhiteKing;
     else if ( board [i][j] == 2 )  image = BlackKing;
     else if ( board [i][j] == 3 )  image = WhiteQueen;
     else if ( board [i][j] == 4 )  image = BlackQueen;
     else if ( board [i][j] == 5 )  image = WhiteRook;
     else if ( board [i][j] == 6 )  image = BlackRook;
     else if ( board [i][j] == 7 )  image = WhiteKnight;
     else if ( board [i][j] == 8 )  image = BlackKnight;
     else if ( board [i][j] == 9 )  image = WhiteBishop;
     else if ( board [i][j] == 10 ) image = BlackBishop;
     else if ( board [i][j] == 11 ) image = WhitePawn;
     else if ( board [i][j] == 12 ) image = BlackPawn;
     else if ( board [i][j] == 13 ) image = WhiteChancellor;
     else if ( board [i][j] == 14 ) image = BlackChancellor;
     else if ( board [i][j] == 15 ) image = WhiteMarshal;
     else if ( board [i][j] == 16 ) image = BlackMarshal;
     else if ( board [i][j] == 31 ) image = WhiteBishop;
     else if ( board [i][j] == 32 ) image = BlackBishop;
     else if ( board [i][j] == 33 ) image = WhiteBishop;
     else if ( board [i][j] == 34 ) image = BlackBishop;

     if (image != null) 
        g.drawImage (image, i*sqsize, (rows-j-1)*sqsize,sqsize,sqsize,this); 
  }
 //--------------------------------------------------------------------------
 // When Imagetracker is used , Java not redrawed all gifs on every repaint()
 public void LoadImages ()
  {
  trk = new MediaTracker(this);

  WhitePawn   = GetImage("pawnw.gif");   trk.addImage(WhitePawn, 0);
  BlackPawn   = GetImage("pawnb.gif");   trk.addImage(BlackPawn, 0);
  WhiteKing   = GetImage("kingw.gif");   trk.addImage(WhiteKing, 0);
  BlackKing   = GetImage("kingb.gif");   trk.addImage(BlackKing, 0);
  WhiteQueen  = GetImage("queenw.gif");  trk.addImage(WhiteQueen, 0);
  BlackQueen  = GetImage("queenb.gif");  trk.addImage(BlackQueen, 0);
  WhiteBishop = GetImage("bishopw.gif"); trk.addImage(WhiteBishop, 0);
  BlackBishop = GetImage("bishopb.gif"); trk.addImage(BlackBishop, 0);
  WhiteKnight = GetImage("knightw.gif"); trk.addImage(WhiteKnight, 0);
  BlackKnight = GetImage("knightb.gif"); trk.addImage(BlackKnight, 0);
  WhiteRook   = GetImage("rookw.gif");   trk.addImage(WhiteRook, 0);
  BlackRook   = GetImage("rookb.gif");   trk.addImage(BlackRook, 0);
    // Special pieces are not loaded automatically!
  WhiteChancellor= GetImage("chancellorw.gif");trk.addImage(WhiteChancellor,0);
  BlackChancellor= GetImage("chancellorb.gif");trk.addImage(BlackChancellor,0);
  WhiteMarshal = GetImage("marshalw.gif");     trk.addImage(WhiteMarshal, 0);
  BlackMarshal = GetImage("marshalb.gif");     trk.addImage(BlackMarshal, 0);
  try {  //Start downloading the images. Wait until they're loaded.
     trk.waitForAll();
  } catch (InterruptedException e) {};
 }
 Image GetImage (String s)
 { Image im;
   try { im = getImage(new URL(getDocumentBase(), s)); }
   catch (MalformedURLException e){ im = null;}
   return im;
 }
 //--------------------------------------------------------------------
 // ForsytheToBoard gets a string that gives a chessboard in Forsythe 
 // notation and puts in output the chessboard, using the conventions
 // (see 0-16 and 31-34 at the start)
 public void ForsytheToBoard ()
 {
  int parseplace = 0; // parseplace is position in input string where we read.
  int x = 0;
  int y = rows-1;
  while ( parseplace < thesetup.length())
  { char inputparseplace = thesetup.charAt(parseplace);
    if (inputparseplace == '/') {x=0; y--;}
    else if (inputparseplace == '-') {board[x][y] = 1000; x++;}
    else if ('1' <= inputparseplace && inputparseplace <= '9')
	 { 
	    int toskip = inputparseplace - '0' ;
	    if (parseplace + 1 < thesetup.length())
	    { char nextchar = thesetup.charAt(parseplace + 1);
	      if ('0' <= nextchar && nextchar <= '9')
	      {	parseplace ++ ; // skip and the next is done at the end
		toskip = 10 * toskip + nextchar - '0';
              }
            }
            x+= toskip;
          }
    else if (x < columns && y  > -1)
	 { board[x][y] = Char2Piece(inputparseplace); x++;
	 }
    parseplace++;
  }
 }
 // gives number code for piece letter from Forsythe notation
 public static int Char2Piece (char letter)
 {
  if (letter == 'K') return 1;
  if (letter == 'k') return 2;
  if (letter == 'Q') return 3;
  if (letter == 'q') return 4;
  if (letter == 'R') return 5;
  if (letter == 'r') return 6;
  if (letter == 'N') return 7;
  if (letter == 'n') return 8;
  if (letter == 'B') return 9;
  if (letter == 'b') return 10;
  if (letter == 'P') return 11;
  if (letter == 'p') return 12;
  if (letter == 'C') return 13;
  if (letter == 'c') return 14;
  if (letter == 'M') return 15;
  if (letter == 'm') return 16;
  if (letter == 'L') return 31;
  if (letter == 'l') return 32;
  if (letter == '-') return 1000;

  return 0;         // default: something went wrong so empty square 
 }
 // ==========================================
 public void ColorBoard ()
 {
  for (int i=0; i<columns; i++)
   for (int j=0; j<rows; j++)
     sqcolor[i][j] = ((i+j) % 2 == 0) ?  'B' : 'b'; 
 }
 public void ColorBoard (String input)
 {
  ColorBoard();
  if( input == null)
     return;
  int parseplace = 0; // parseplace is position in input string where we read.
  int x = 0;
  int y = rows-1;
  while (parseplace < input.length())
  { char inputparseplace = input.charAt(parseplace);
    if ( (inputparseplace >= 'A' && inputparseplace <= 'Z') ||
         (inputparseplace >= 'a' && inputparseplace <= 'z') )
       {
	 if ( x < columns && y > -1  )
	 {   sqcolor[x][y] = inputparseplace; x++; }
       }
    else if ('1' <= inputparseplace && inputparseplace <= '9')
	 { 
	    int toskip = inputparseplace - '0' ;
	    if (parseplace + 1 < input.length())
	    { char nextchar = input.charAt(parseplace + 1);
	      if ('0' <= nextchar && nextchar <= '9')
	      {	parseplace ++;
		toskip = 10 * toskip + nextchar - '0';
              }
            }
            x+= toskip;
         }
    else if (inputparseplace == '/')
	 { 
	   x=0;	 y--;
         }
    parseplace++;
  }
 }
 public void DisableCastling() {  control.DisableCastling();}
 public void DisableDoublePawnStep() { control.DisableDoublePawnStep();}
 public void DisableDoublePawnStep(int player)
            { control.DisableDoublePawnStep(player); }
 // Interface with computer player
 public void MakeComputerMove()
 {
  moveNumber++;
  setText("Black move");
  Move computermove = ComputerPlayer.findmove(this,control,computercolor);
  if (!computermove.isvoid())	    control.DoMove(computermove);

  if(control.Mated(1-computercolor))           setText("BLACK WIN: MATE!!!");
  else if(control.InCheck(1-computercolor))    setText("White in check!!!"); 
  else if(control.Stalemated(1-computercolor)) setText("STALEMATE !!!");
  else if(control.Mated(computercolor))        setText("WHITE WIN: MATE!!!"); 
  else if(control.Stalemated(computercolor))   setText("STALEMATE  !!!");
  else                                         setText("White move");
 }
}
///////////////////////////////////////////////////////////////////////
class Move
{
  int startrow, endrow, startcol, endcol;
  Move()
  {  startrow = endrow = startcol = endcol = 0;
  }
  Move(int k1, int r1, int k2, int r2)
  {  startcol = k1; startrow = r1; endcol = k2; endrow = r2;
  }
  boolean isvoid()   // Return true if void move (all zeros)
  {  return (startrow+endrow+startcol+endcol)==0 ?  true : false;
  }
}
///////////////////////////////////////////////////////////////////////
class ComputerPlayer
{
 static int plydepth=2;
 // Overloading: findmove tries to find the best move for given position
 // ChessMoveChecker and Move: evaluate how good this particular move is.
 public static Move findmove (ChessCanvas parent,
                              ChessMoveChecker position, int player)
 {
  Move themove = new Move();
  int bestmove = -50000;
  for (int i1=0; i1< position.columns; i1++)
   for (int j1=0; j1 <  position.rows; j1++)
    if (!position.EmptySquare(i1,j1))
     if (position.PieceOwner(i1,j1)==player)
      for (int i2=0; i2< position.columns; i2++)
       for (int j2=0; j2< position.rows; j2++)
	if (position.SemiLegalMove(i1,j1,i2,j2))
	{
          ChessMoveChecker aftermove = new ChessMoveChecker(position);
          aftermove.DoMove(i1,j1,i2,j2);
          if ( aftermove.InCheck(player) ) continue;
          int tryevaluate = Evaluate(aftermove,plydepth -1);
          if (tryevaluate > bestmove)
          {   themove = new Move(i1,j1,i2,j2); 
              bestmove = tryevaluate;
          }
        }
  return themove;
 }
 // This one gives a value for the player that moved to this position.
 static int Evaluate (ChessMoveChecker position, int plies)
 {
  if (plies <=0) return Evaluate(position);
  int otherplayer = 1-position.lastplayermoved;
    // what is the best move for the other player?
  int bestmove = -50000;
  for (int i1=0; i1<position.columns; i1++)
   for (int j1=0; j1 < position.rows; j1++)
    if (!position.EmptySquare(i1,j1))
     if (position.PieceOwner(i1,j1)==otherplayer)
      for (int i2=0; i2<position.columns; i2++)
       for (int j2=0; j2<position.rows; j2++)
	if (position.SemiLegalMove(i1,j1,i2,j2))
	{
          ChessMoveChecker aftermove = new ChessMoveChecker(position);
          aftermove.DoMove(i1,j1,i2,j2);
          if ( aftermove.InCheck(otherplayer) ) continue;
          int tryevaluate = Evaluate(aftermove,plies -1);
          if (tryevaluate > bestmove)
              bestmove = tryevaluate;
        }
  return 1-bestmove;
 }
 static int Evaluate (ChessMoveChecker position)
 {
   int otherplayer = 1- position.lastplayermoved;
   if (position.Mated(otherplayer))      return 50000; // Mate : value is high
   if (position.Stalemated(otherplayer)) return 0; // Stalemate : draw is zero
       //compute the difference in value of the pieces
   int totvalue = diffvalueofpieces(position,1-otherplayer);
   int ran = 1+ (int) (Math.random() * 10);  // Random from 1 to 10
   return ran + 10*totvalue;
 }
 // what is the value of the pieces of player minus the value of its opponent?
 static int diffvalueofpieces (ChessMoveChecker position, int player)
 {
   int totvalue = 0;
   for (int k=0; k<position.columns; k++)
    for (int r=0; r<position.rows;r++)
     if (!position.EmptySquare(k,r))
     {
      if (position.PieceOwner(k,r)==player)
       totvalue += PieceValue(position.board[k][r]);
      else
       totvalue -= PieceValue(position.board[k][r]);
     }
   return totvalue;
 }
 static int PieceValue(int piece) // give value of piece, notated as integer
 {  switch (piece)
    {
     default:          return 0;
     case 1:  case 2:   return 1000; // king. Who cares what value he gets.
     case 3:  case 4:   return 80;   // queen
     case 5:  case 6:   return 50;   // rook
     case 7:  case 8:   return 30;   // knight
     case 9:  case 10:  return 30;   // bishop
     case 11: case 12:  return 10;   // pawn
     case 13: case 14:  return 80;      // chancellor
     case 15:  case 16: return 60;      // marshal
     case 31: case 32:  return 28;      // bcr - try to develop them quickly
     case 33: case 34:  return 27;      // bcr
    }
 }
}
////////////////////////////////////////////////////////////////////////////
class BoardState
{
  int[][] board;
  boolean whiteshortcastling, blackshortcastling;
  boolean whitelongcastling,  blacklongcastling;
  int lastplayermoved; // Which player moved last : 0 - white, 1 - black
  Move lastmove;
     // Copy ChessMoveChecker() status to stack for back move;
  BoardState (ChessMoveChecker tocopy)
  {
    whiteshortcastling = tocopy.whiteshortcastling;
    blackshortcastling = tocopy.blackshortcastling;
    whitelongcastling  = tocopy.whitelongcastling;
    blacklongcastling  = tocopy.blacklongcastling;
        // As white starts, that would correspond to black playing last
    lastplayermoved  = tocopy.lastplayermoved;
    lastmove = new Move ( tocopy.lastmove.startcol, tocopy.lastmove.startrow,
                          tocopy.lastmove.endcol,   tocopy.lastmove.endrow);   
        // Initiate current board according to parent board
    board = new int[tocopy.columns][tocopy.rows];
    for (int i = 0; i< tocopy.columns; i++)
      for (int j=0; j< tocopy.rows; j++)
       board[i][j]=tocopy.board[i][j];
  }
}
//////////////////////////////////////////////////////////////////////////
class ChessMoveChecker
{
  int[][] board;
  static int rows, columns;
  static ChessCanvas thecanvas;
  static int kingrw; // position that white king starts: ( kingrw, 0)
  static int kingrb; // position that black king starts: ( kingrb, rows - 1)
          // booleans that tell which player may castle;
  boolean whiteshortcastling, blackshortcastling;
  boolean whitelongcastling,  blacklongcastling;
         // is it legal in this variant to make a double pawn step?
  boolean whitedoublepawnstep, blackdoublepawnstep;

  int lastplayermoved; // Which player moved last : 0 - white, 1 - black
  Move lastmove;

// Constructor method
ChessMoveChecker (ChessCanvas hetboard)
  {
    board   = hetboard.board;   // Copy pointer, not board itself
    rows    = hetboard.rows;
    columns = hetboard.columns;
    thecanvas = hetboard;       // Set pointer to board of parent
        // Sets the booleans whether one can castle.
    whiteshortcastling  = true;
    blackshortcastling  = true;
    whitelongcastling   = true;
    blacklongcastling   = true;
    whitedoublepawnstep = true;
    blackdoublepawnstep = true;
        // As white starts, that would correspond to black playing last
    lastplayermoved = 1;
    lastmove = new Move();

    SetKingPosition();   // Sets castling squares right
}
// sets variables kingrw and kingrb for castling info
public void SetKingPosition()
{   // Find the place where the king starts for castling
    // Standard chess setup:
    kingrw = 4;
    kingrb = 4;

    for (int i=0; i< columns; i++)
      if ( board[i][0]==1 )       kingrw = i ; 
    for (int i=0; i< columns; i++)
      if ( board[i][rows-1]==2 )  kingrb = i ; 
}
// 2nd constructor method: copies parent's board
  ChessMoveChecker (ChessMoveChecker tocopy)
  {
    whiteshortcastling = tocopy.whiteshortcastling;
    blackshortcastling = tocopy.blackshortcastling;
    whitelongcastling  = tocopy.whitelongcastling;
    blacklongcastling  = tocopy.blacklongcastling;
        // As white starts, that would correspond to black playing last
    lastplayermoved  = tocopy.lastplayermoved;
    lastmove = new Move ( tocopy.lastmove.startcol, tocopy.lastmove.startrow,
                          tocopy.lastmove.endcol,   tocopy.lastmove.endrow); 
        // Initiate current board according to parent board
    board = new int[columns][rows];
    for (int i = 0; i< columns; i++)
      for (int j=0; j< rows; j++)
        board[i][j]=tocopy.board[i][j];
  }
// Set board from stack
public void SetBoard (BoardState tocopy)
{
    whiteshortcastling = tocopy.whiteshortcastling;
    blackshortcastling = tocopy.blackshortcastling;
    whitelongcastling  = tocopy.whitelongcastling;
    blacklongcastling  = tocopy.blacklongcastling;
        // As white starts, that would correspond to black playing last
    lastplayermoved  = tocopy.lastplayermoved;
    lastmove = new Move ( tocopy.lastmove.startcol, tocopy.lastmove.startrow,
                          tocopy.lastmove.endcol,   tocopy.lastmove.endrow);   
        // Initiate current board according to parent board
    for (int i = 0; i< columns; i++)
      for (int j=0; j< rows; j++)
       board[i][j]=tocopy.board[i][j];
}
// test whether square (kolom, row) is a square of the board
boolean LegalSquare (int kolom, int row)
  {   // check if inside board dimensions
   if (kolom < 0)       return false;
   if (kolom > columns) return false;
   if (row < 0)         return false;
   if (row > rows)      return false;
     // check if not a white, i.e., non-existing square
   if ( board[kolom][row] == 1000) return false;
   return true;
  }
// test of move van veld (k1, r1) naar (k2, r2) legaal is
public boolean LegalMove (int k1, int r1, int k2, int r2)
  {
     if (!SemiLegalMove(k1,r1,k2,r2))    return false;
     if (InCheckAfterMove(k1,r1,k2,r2))  return false;
     return true;
  }
public boolean LegalMove (Move m)
  {
    return LegalMove(m.startcol,m.startrow,m.endcol,m.endrow);
  }
// A move is SemiLegal if it is legal without looking for check
public boolean SemiLegalMove (int k1, int r1, int k2, int r2)
  {
    if (!LegalSquare(k2,r2))                 return false;
    if (k1 == k2 && r1 == r2)                return false;
    if (TakingPieceOfSameColor(k1,r1,k2,r2)) return false;
    switch (board[k1][r1]) 
    {
      case 0: case 1000: return false;
      case 1:            return LegalWhiteKingMove (k1,r1,k2,r2);
      case 2:            return LegalBlackKingMove (k1,r1,k2,r2);
      case 3:  case 4:   return LegalQueenMove     (k1,r1,k2,r2);
      case 5:  case 6:   return LegalRookMove      (k1,r1,k2,r2);
      case 7:  case 8:   return LegalKnightMove    (k1,r1,k2,r2);
      case 9:  case 10:  return LegalBishopMove    (k1,r1,k2,r2);
      case 11:           return LegalWhitePawnMove (k1,r1,k2,r2);
      case 12:           return LegalBlackPawnMove (k1,r1,k2,r2);
      case 13:  case 14: return LegalChancellorMove(k1,r1,k2,r2);
      case 15:  case 16: return LegalMarshalMove   (k1,r1,k2,r2);
      case 31:  case 32: return LegalUnconvertedBishopMove (k1,r1,k2,r2);
      case 33:  case 34: return LegalWazirMove     (k1,r1,k2,r2);
    }
    return true;
  }
boolean LegalWhiteKingMove (int k1,int r1, int k2, int r2)
{     // check on castling.
  if (k1==kingrw && k2==kingrw+2 && r1==0 && r2==0)
    return LegalWhiteShortCastling();
  if (k1==kingrw && k2==kingrw-2 && r1==0 && r2==0)
    return LegalWhiteLongCastling();
  return LegalKingMove(k1,r1,k2,r2);
}
boolean LegalBlackKingMove (int k1,int r1,int k2,int r2)
{     // check on castling.
  if (k1==kingrw && k2==kingrw+2 && r1==rows-1 && r2==rows-1)
    return LegalBlackShortCastling();
  if (k1==kingrw && k2==kingrw-2 && r1==rows-1 && r2==rows-1)
    return LegalBlackLongCastling();
  return LegalKingMove(k1,r1,k2,r2);
}
boolean LegalKingMove (int k1,int r1,int k2,int r2)
{
   // this precisely tells if one is trying to make a kingsmove
   return ( (k1-k2)*(k1-k2) <=1 && (r1-r2)*(r1-r2) <= 1);
}
boolean LegalRookMove (int k1, int r1, int k2, int r2)
{
   if ( k1 == k2 )
   {
      for (int i= Math.min(r1,r2)+1; i < Math.max(r1,r2); i++)
          if ( !EmptySquare(k1,i)) return false;
      return true;
   }
   else if (r1 == r2)
   {
      for (int i= Math.min(k1,k2)+1; i < Math.max(k1,k2); i++)
          if ( !EmptySquare(i,r1)) return false;
      return true;
   }
   else return false;
}
boolean LegalKnightMove (int k1, int r1, int k2, int r2)
{
 return ( Math.abs((k1-k2)*(r1-r2)) ==2 ) ? true : false ; 
}
boolean LegalBishopMove (int k1, int r1, int k2, int r2)
{
   int kdif = k1-k2;
   int rdif = r1-r2;
   if (kdif == rdif || kdif == -rdif)
   {
     int kstep = (k1 < k2) ? 1 : -1;
     int rstep = (r1 < r2) ? 1 : -1;
     int i=k1+kstep; 
     int j=r1+rstep;
     while ( i != k2 )
     {
       if (!EmptySquare(i,j)) return false;
       i += kstep;
       j += rstep;
     }
     return true;
   }
   else return false;
}
boolean LegalUnconvertedBishopMove (int k1, int r1, int k2, int r2)
{
   return (LegalBishopMove(k1,r1,k2,r2) || LegalWazirMove(k1,r1,k2,r2));
}
boolean LegalQueenMove (int k1, int r1, int k2, int r2)
{
   return (LegalRookMove(k1,r1,k2,r2) || LegalBishopMove(k1,r1,k2,r2));
}
boolean LegalChancellorMove (int k1, int r1, int k2, int r2)
{
   return (LegalRookMove(k1,r1,k2,r2) || LegalKnightMove(k1,r1,k2,r2));
}
// Marshal moves like Bishop or Knight
boolean LegalMarshalMove (int k1,int r1, int k2, int r2)
{
   return (LegalBishopMove(k1,r1,k2,r2) || LegalKnightMove(k1,r1,k2,r2));
}
// Wazir: moves one square vertically or horizontally
boolean LegalWazirMove (int k1, int r1, int k2, int r2)
{
 return ( Math.abs(k1-k2) + Math.abs(r1-r2) == 1 );
}
boolean LegalWhitePawnMove (int k1, int r1, int k2, int r2)
{    // no en passant capture yet
  boolean taking = (board[k2][r2] == 0 ) ? false : true;
   if (taking == true && r2 == r1 + 1 && ( (k1 == k2 +1) || (k1 == k2 -1)))
        return true;
   else if (taking == false && (r2 == r1 + 1) && k1 == k2)
	return true;
   else if (taking == false && r1 == 1 && r2 == 3 && k1 == k2 &&
	    board[k1][2] == 0 )
	return whitedoublepawnstep;
   else if (LegalWhitePawnEnPassantCapture (k1, r1, k2, r2))
        return true;
   return false;
}
boolean LegalBlackPawnMove (int k1, int r1, int k2, int r2)
{
  boolean taking = (board[k2][r2] == 0 ) ? false : true;
  if (taking == true && r2 == r1 - 1 && ( (k1 == k2 +1) || (k1 == k2 -1)))
        return true;
   else if (taking == false && r2 == r1 - 1 && k1 == k2)
	return true;
   else if (taking == false && r1 == rows-2 && r2 == r1-2 && k1 == k2 &&
	    board[k1][r1-1] == 0 )
	return blackdoublepawnstep;
   else if (LegalBlackPawnEnPassantCapture (k1, r1, k2, r2))
        return true;
   return false;
}
public boolean LegalWhitePawnEnPassantCapture (int k1, int r1, int k2, int r2)
{
  if (board[k1][r1] != 11)  return false;  // white pawn on starting square
  if (board[k2][r2] != 0 )  return false;  // go to empty square
  // check if opponent made a pawn double step on destination column
  if (lastmove.startcol != k2) return false;
  if (lastmove.endcol != k2)   return false;
  if (lastmove.startrow != lastmove.endrow +2) return false;
  if (board[lastmove.endcol][lastmove.endrow] != 12) return false;
  if (r1 != rows-4) return false;
  if (r2 != rows-3) return false;
  if (Math.abs(k1-k2) != 1) return false;
  return true;
}
public boolean LegalBlackPawnEnPassantCapture (int k1, int r1, int k2, int r2)
{
  if (board[k1][r1] != 12)  return false;  // white pawn on starting square
  if (board[k2][r2] != 0 )  return false;  // go to empty square
  // check if opponent made a pawn double step on destination column
  if (lastmove.startcol != k2) return false;
  if (lastmove.endcol != k2)   return false;
  if (lastmove.startrow != lastmove.endrow -2)  return false;
  if (board[lastmove.endcol][lastmove.endrow] != 11) return false;
  if (r1 != 3)  return false;
  if (r2 != 2)  return false;
  if (Math.abs(k1-k2) != 1) return false;
  return true;
}
// tests if square (col,row) is empty
boolean EmptySquare (int col, int row)
{
  return (board[col][row] == 0) ? true : false;
}
// true if one tries to take a piece of the same color
boolean TakingPieceOfSameColor (int k1, int r1, int k2, int r2)
{
   if (EmptySquare (k2,r2)) return false;
   return ( (board[k1][r1] % 2) == (board[k2][r2] % 2)) ? true : false ;
}
//--------------------------------------------
// Methods about castling
void ChangeCastlingStatus (int k1, int r1, int k2, int r2)
{    // if a square moves from a corner, one type of castling is never
     // possible anymore.
   if (k1==0 && r1 == 0)             whitelongcastling  = false;
   if (k1==columns-1 && r1 == 0)     whiteshortcastling = false;
   if (k1==0 && r1 == rows-1)        blacklongcastling  = false;
   if (k1==columns-1 && r1 ==rows-1) blackshortcastling = false;
   if (board[k1][r1] == 1) // white king moves
       whitelongcastling  = whiteshortcastling = false;
   if (board[k1][r1] == 2) // black king moves
       blacklongcastling  = blackshortcastling = false;
}
// Is it legal for white to castle short?
boolean LegalWhiteShortCastling()
{
  if (InCheck(0)) return false;
  for (int i= kingrw + 1; i < columns -1; i++)
  { 
    if (!EmptySquare(i,0)) return false;
    if (Attacked(1,i,0))   return false;
  }
  return whiteshortcastling;
}
// is it legal for white to castle long?
boolean LegalWhiteLongCastling()
{
  if (InCheck(0)) return false;
  for (int i= kingrw - 1; i >0; i--)
  { 
    if (!EmptySquare(i,0))  return false;
    if (Attacked(1,i,0))    return false;
  }
  return whitelongcastling;
}
  // is it legal for black to castle long?
boolean LegalBlackLongCastling()
{
  if (InCheck(1)) return false;
  for (int i= kingrw - 1; i >0; i--)
  { 
    if (!EmptySquare(i,rows-1)) return false;
    if (Attacked(0,i,rows-1))   return false;
  }
  return blacklongcastling;
}
 // not yet finished: check for check
boolean LegalBlackShortCastling()
{
  if (InCheck(1)) return false;
    // is it legal for black to castle short?
  for (int i= kingrb + 1; i < columns -1; i++)
  { 
    if (!EmptySquare(i,rows-1))  return false;
    if (Attacked(0,i,rows-1))    return false;
  }
  return blackshortcastling;
}
//------------------------------------
// Determine if the move is a pawn promotion.
boolean DeterminePawnPromotion (int k1, int r1, int k2, int r2)
{
  if (board[k1][r1]==11 && r2 == rows-1)  return true;
  if (board[k1][r1]==12 && r2 == 0)       return true;
  return false;
}
public void SetLastPlayerMoved(int i)
{ 
  lastplayermoved = i;
}
public int LastPlayerMoved()
{ 
  return lastplayermoved;
}
// gets number of piece as input.
// returns 0 if white owns the piece, 1 - if black
// not specified if this is not a piece.
public int PieceOwner(int i)
{
   return 1-(i%2);
}
// returns the owner of the piece on square (k,r)
public int PieceOwner(int k, int r)
{
   return PieceOwner(board[k][r]);
}
// checks if the right player wants to make a move
public boolean RightPlayer(int k1, int r1)
{
   return (PieceOwner(k1,r1) != LastPlayerMoved());
}
// Checks if player is in check
boolean InCheck (int player)
{
// first find the king piece of the player.
  int kingpiece = 1 + player;
  for (int k=0; k< columns; k++)
    for (int r=0; r < rows; r++)
       if (board[k][r]==kingpiece)
	 // crude: check if any piece could move to the kings position
	 for (int k2=0; k2<columns; k2++)
	   for (int r2=0; r2<rows; r2++)
	     if ((board[k2][r2] != 0) || (board[k2][r2] !=1000))
	       if (SemiLegalMove(k2,r2,k,r)) 
		 return true;
  return false;
}
// has player a semilegal move that attacks (k,r)
boolean Attacked (int player, int k, int r)
{   // unwritten
 for (int k2=0; k2 < columns; k2++)
   for (int r2=0; r2 < rows; r2++)
     if ((! EmptySquare(k2,r2)) || (board[k2][r2]!=1000) )
	if (PieceOwner(k2,r2)== player)
	   if (SemiLegalMove(k2,r2,k,r))
	      return true;
 return false;
}
boolean InCheckAfterMove (int k1, int r1, int k2, int r2)
{
  int player = PieceOwner(k1,r1);
  ChessMoveChecker aftermove = new ChessMoveChecker(this);
  aftermove.DoMove(k1,r1,k2,r2);
  return aftermove.InCheck(player);
}
void DoMove (Move m)
  {
      DoMove(m.startcol,m.startrow,m.endcol,m.endrow);
  }
void DoMove (int k1, int r1, int k2, int r2)
{
      // First determine if this is a promotion move
   boolean promotion = DeterminePawnPromotion(k1,r1,k2,r2);
     // determine if this is a castling move and enpassant
   boolean castled =((board[k1][r1]>2) || (Math.abs(k1-k2)<2)) ? false : true;
   boolean whiteenpassant = LegalWhitePawnEnPassantCapture (k1, r1, k2, r2);
   boolean blackenpassant = LegalBlackPawnEnPassantCapture (k1, r1, k2, r2);
     // update status
   lastmove = new Move (k1,r1, k2, r2); // sets the last move
   SetLastPlayerMoved(PieceOwner(k1,r1));
   ChangeCastlingStatus(k1,r1,k2,r2);
     // do the move
   board[k2][r2] = board[k1][r1];
   board[k1][r1]=0;

    // additional work if promotion, castling, enpassant
   if (promotion)       board[k2][r2] = DeterminePromotionPiece(k2,r2);
   if (castled)         HandleCastling(k1,r1,k2,r2);
   if (whiteenpassant)  board[k2][r2-1]=0;  // remove captured pawn
   if (blackenpassant)  board[k2][r2+1]=0;  // remove captured pawn

   HandleBCR(k1,r1,k2,r2);
}
// Handle the bishop conversion rule.
void HandleBCR(int k1, int r1, int k2, int r2)
{
 for (int color=0; color < 2; color++) 
  if (board[k2][r2] == (31+color)) // unconverted bishop
  {
    board[k2][r2] = 9+color;    // change the bishop to a normal bishop
    // Test if this was a Wazir Move
    // Find a possible other unconverted bishop and make it a 9,33, 10 or 34
    for (int i=0; i<columns; i++)
     for (int j=0; j<rows; j++)
       if (board[i][j]==(31+color))
	 board[i][j]= LegalWazirMove(k1,r1,k2,r2) ?  9+color : 33+color;
   }
 // change wazir and then bishop to bishop
 if (board[k2][r2] == 33) board[k2][r2]=9;
 if (board[k2][r2] == 34) board[k2][r2]=10;
}
// modify later for including Fisher Random Chess.
void HandleCastling(int k1,int r1,int k2,int r2)
{
   if (k1 > k2)                           // castling to left of board
   { board[k2+1][r2] = board[0][r2];
     board[0][r2] = 0;
   }  else                                // castling to right of board
   { board[k2-1][r2] = board[columns -1][r2];
     board[columns-1][r2]=0;
   }
}
// Determinate pawn promoted piece (always queen at this stage)
int DeterminePromotionPiece (int k, int r) 
{
  return 3+PieceOwner(k,r);    // Promoted to queen 
}
//================== Methods for variants
public void DisableCastling()
{
  whiteshortcastling = false;
  blackshortcastling = false;
  whitelongcastling = false;
  blacklongcastling = false;
}
public void DisableDoublePawnStep()
{
  whitedoublepawnstep = false;
  blackdoublepawnstep = false;
}
public void DisableDoublePawnStep(int player)
{
  if (player == 0) whitedoublepawnstep = false;
  if (player == 1) blackdoublepawnstep = false;
}
boolean Mated(int player)
{
  if (lastplayermoved != player)
    if (InCheck(player))
      return (!HasALegalMove(player));
  return false;
}
boolean Stalemated(int player)
{
  if (lastplayermoved != player)
    if (!InCheck(player))
      return (!HasALegalMove(player));
  return false;
}
boolean HasALegalMove(int player)
{
  for (int k=0; k<columns; k++)
    for (int r=0; r<rows;r++)
     if ((!EmptySquare(k,r)) || (board[k][r] != 1000))
      if (PieceOwner(k,r)==player)
       for (int i2=0; i2<columns; i2++)
        for (int j2=0; j2<rows; j2++)
	     if (LegalMove(k,r,i2,j2))
	        return true;
   return false;
}
}
